@@ -11,9 +11,9 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2000, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2011, Live Networks, Inc.  All rights reserved
 //	Help by Carlo Bonamico to get working for Windows
 // Delay queue
 // Implementation
@@ -33,7 +33,7 @@ int Timeval::operator>=(const Timeval& arg2) const {
 
 void Timeval::operator+=(const DelayInterval& arg2) {
   secs() += arg2.seconds(); usecs() += arg2.useconds();
-  if (usecs() >= MILLION) {
+  if (useconds() >= MILLION) {
     usecs() -= MILLION;
     ++secs();
   }
@@ -41,23 +41,24 @@ void Timeval::operator+=(const DelayInterval& arg2) {
 
 void Timeval::operator-=(const DelayInterval& arg2) {
   secs() -= arg2.seconds(); usecs() -= arg2.useconds();
-  if (usecs() < 0) {
+  if ((int)useconds() < 0) {
     usecs() += MILLION;
     --secs();
   }
-  if (secs() < 0)
+  if ((int)seconds() < 0)
     secs() = usecs() = 0;
+
 }
 
 DelayInterval operator-(const Timeval& arg1, const Timeval& arg2) {
   time_base_seconds secs = arg1.seconds() - arg2.seconds();
   time_base_seconds usecs = arg1.useconds() - arg2.useconds();
-  
-  if (usecs < 0) {
+
+  if ((int)usecs < 0) {
     usecs += MILLION;
     --secs;
   }
-  if (secs < 0)
+  if ((int)secs < 0)
     return DELAY_ZERO;
   else
     return DelayInterval(secs, usecs);
@@ -69,11 +70,11 @@ DelayInterval operator-(const Timeval& arg1, const Timeval& arg2) {
 DelayInterval operator*(short arg1, const DelayInterval& arg2) {
   time_base_seconds result_seconds = arg1*arg2.seconds();
   time_base_seconds result_useconds = arg1*arg2.useconds();
-  
+
   time_base_seconds carry = result_useconds/MILLION;
   result_useconds -= carry*MILLION;
   result_seconds += carry;
-  
+
   return DelayInterval(result_seconds, result_useconds);
 }
 
@@ -112,7 +113,11 @@ DelayQueue::DelayQueue()
 }
 
 DelayQueue::~DelayQueue() {
-  while (fNext != this) removeEntry(fNext);
+  while (fNext != this) {
+    DelayQueueEntry* entryToRemove = fNext;
+    removeEntry(entryToRemove);
+    delete entryToRemove;
+  }
 }
 
 void DelayQueue::addEntry(DelayQueueEntry* newEntry) {
@@ -123,9 +128,9 @@ void DelayQueue::addEntry(DelayQueueEntry* newEntry) {
     newEntry->fDeltaTimeRemaining -= cur->fDeltaTimeRemaining;
     cur = cur->fNext;
   }
-  
+
   cur->fDeltaTimeRemaining -= newEntry->fDeltaTimeRemaining;
-  
+
   // Add "newEntry" to the queue, just before "cur":
   newEntry->fNext = cur;
   newEntry->fPrev = cur->fPrev;
@@ -134,7 +139,7 @@ void DelayQueue::addEntry(DelayQueueEntry* newEntry) {
 
 void DelayQueue::updateEntry(DelayQueueEntry* entry, DelayInterval newDelay) {
   if (entry == NULL) return;
-  
+
   removeEntry(entry);
   entry->fDeltaTimeRemaining = newDelay;
   addEntry(entry);
@@ -147,7 +152,7 @@ void DelayQueue::updateEntry(long tokenToFind, DelayInterval newDelay) {
 
 void DelayQueue::removeEntry(DelayQueueEntry* entry) {
   if (entry == NULL || entry->fNext == NULL) return;
-  
+
   entry->fNext->fDeltaTimeRemaining += entry->fDeltaTimeRemaining;
   entry->fPrev->fNext = entry->fNext;
   entry->fNext->fPrev = entry->fPrev;
@@ -175,7 +180,7 @@ void DelayQueue::handleAlarm() {
     // This event is due to be handled:
     DelayQueueEntry* toRemove = head();
     removeEntry(toRemove); // do this first, in case handler accesses queue
-    
+
     toRemove->handleTimeout();
   }
 }
@@ -193,9 +198,14 @@ DelayQueueEntry* DelayQueue::findEntryByToken(long tokenToFind) {
 void DelayQueue::synchronize() {
   // First, figure out how much time has elapsed since the last sync:
   EventTime timeNow = TimeNow();
+  if (timeNow < fLastSyncTime) {
+    // The system clock has apparently gone back in time; reset our sync time and return:
+    fLastSyncTime  = timeNow;
+    return;
+  }
   DelayInterval timeSinceLastSync = timeNow - fLastSyncTime;
   fLastSyncTime = timeNow;
-  
+
   // Then, adjust the delay queue for any entries whose time is up:
   DelayQueueEntry* curEntry = head();
   while (timeSinceLastSync >= curEntry->fDeltaTimeRemaining) {
@@ -217,8 +227,4 @@ EventTime TimeNow() {
   return EventTime(tvNow.tv_sec, tvNow.tv_usec);
 }
 
-DelayInterval TimeRemainingUntil(const EventTime& futureEvent) {
-  return futureEvent - TimeNow();
-}
-
 const EventTime THE_END_OF_TIME(INT_MAX);
