@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A filter that breaks up an AC3 audio elementary stream into frames
 // Implementation
 
@@ -44,9 +44,8 @@ public:
   virtual ~AC3AudioStreamParser();
 
 public:
-  Boolean testStreamCode(unsigned char ourStreamCode,
-			 unsigned char* ptr, unsigned size);
-     // returns True iff the initial stream code is ours
+  void testStreamCode(unsigned char ourStreamCode,
+		      unsigned char* ptr, unsigned size);
   unsigned parseFrame(unsigned& numTruncatedBytes);
      // returns the size of the frame that was acquired, or 0 if none was
 
@@ -151,13 +150,8 @@ void AC3AudioStreamFramer
 
 void AC3AudioStreamFramer
 ::handleNewData(unsigned char* ptr, unsigned size) {
-  if (!fParser->testStreamCode(fOurStreamCode, ptr, size)) {
-    // This block of data is not for us; try again:
-    parseNextFrame();
-    return;
-  }
+  fParser->testStreamCode(fOurStreamCode, ptr, size);
 
-  // Now that we know that this data is for us, get the next frame:
   parseNextFrame();
 }
 
@@ -167,7 +161,7 @@ void AC3AudioStreamFramer::parseNextFrame() {
     // We were able to acquire a frame from the input.
     // It has already been copied to the reader's space.
     fFrameSize = acquiredFrameSize;
-    
+
     // Also set the presentation time, and increment it for next time,
     // based on the length of this frame:
     fPresentationTime = fNextFramePresentationTime;
@@ -238,23 +232,21 @@ void AC3AudioStreamParser::registerReadInterest(unsigned char* to,
   fMaxSize = maxSize;
 }
 
-Boolean AC3AudioStreamParser
+void AC3AudioStreamParser
 ::testStreamCode(unsigned char ourStreamCode,
 		 unsigned char* ptr, unsigned size) {
-  if (size < 4) return False; // shouldn't happen
+  if (ourStreamCode == 0) return; // we assume that there's no stream code at the beginning of the data
+
+  if (size < 4) return; // shouldn't happen
   unsigned char streamCode = *ptr;
 
   if (streamCode == ourStreamCode) {
     // Remove the first 4 bytes from the stream:
     memmove(ptr, ptr + 4, size - 4);
     totNumValidBytes() = totNumValidBytes() - 4;
-
-    return True;
   } else {
     // Discard all of the data that was just read:
     totNumValidBytes() = totNumValidBytes() - size;
-    
-    return False;
   }
 }
 
@@ -270,7 +262,7 @@ unsigned AC3AudioStreamParser::parseFrame(unsigned& numTruncatedBytes) {
 
   try {
     saveParserState();
-    
+
     // We expect an AC3 audio header (first 2 bytes == 0x0B77) at the start:
     while (1) {
       unsigned next4Bytes = test4Bytes();
@@ -280,10 +272,10 @@ unsigned AC3AudioStreamParser::parseFrame(unsigned& numTruncatedBytes) {
     }
     fCurrentFrame.hdr0 = get4Bytes();
     fCurrentFrame.hdr1 = test4Bytes();
-    
+
     fCurrentFrame.setParamsFromHeader();
     fHaveParsedAFrame = True;
-    
+
     // Copy the frame to the requested destination:
     unsigned frameSize = fCurrentFrame.frameSize;
     if (frameSize > fMaxSize) {
@@ -299,7 +291,7 @@ unsigned AC3AudioStreamParser::parseFrame(unsigned& numTruncatedBytes) {
     fTo[3] = fCurrentFrame.hdr0;
     getBytes(&fTo[4], frameSize-4);
     skipBytes(numTruncatedBytes);
-    
+
     return frameSize;
   } catch (int /*e*/) {
 #ifdef DEBUG
@@ -316,7 +308,7 @@ void AC3AudioStreamParser::readAndSaveAFrame() {
 
   fSavedFrameFlag = 0;
   fUsingSource->getNextFrame(fSavedFrame, maxAC3FrameSize,
-			     afterGettingSavedFrame, this, 
+			     afterGettingSavedFrame, this,
 			     onSavedFrameClosure, this);
   fUsingSource->envir().taskScheduler().doEventLoop(&fSavedFrameFlag);
 }
