@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // AMR Audio RTP Sources (RFC 3267)
 // Implementation
 
@@ -101,7 +101,6 @@ private:
   RawAMRRTPSource* fInputSource;
   class AMRDeinterleavingBuffer* fDeinterleavingBuffer;
   Boolean fNeedAFrame;
-
 };
 
 
@@ -480,6 +479,7 @@ void AMRDeinterleaver::doGetNextFrame() {
 }
 
 void AMRDeinterleaver::doStopGettingFrames() {
+  fNeedAFrame = False;
   fInputSource->stopGettingFrames();
 }
 
@@ -537,7 +537,7 @@ void AMRDeinterleavingBuffer
 #ifdef DEBUG
     fprintf(stderr, "AMRDeinterleavingBuffer::deliverIncomingFrame() param sanity check failed (%d,%d,%d,%d)\n", frameSize, fILL, ILP, frameIndex);
 #endif
-    abort();
+    source->envir().internalError();
   }
 
   --frameIndex; // because it was incremented by the source when this frame was read
