@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2005 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A simple UDP source, where every UDP payload is a complete frame
 // Implementation
 
@@ -27,9 +27,15 @@ BasicUDPSource* BasicUDPSource::createNew(UsageEnvironment& env,
 }
 
 BasicUDPSource::BasicUDPSource(UsageEnvironment& env, Groupsock* inputGS)
-  : FramedSource(env), fInputGS(inputGS) {
+  : FramedSource(env), fInputGS(inputGS), fHaveStartedReading(False) {
   // Try to use a large receive buffer (in the OS):
   increaseReceiveBufferTo(env, inputGS->socketNum(), 50*1024);
+
+  // Make the socket non-blocking, even though it will be read from only asynchronously, when packets arrive.
+  // The reason for this is that, in some OSs, reads on a blocking socket can (allegedly) sometimes block,
+  // even if the socket was previously reported (e.g., by "select()") as having data available.
+  // (This can supposedly happen if the UDP checksum fails, for example.)
+  makeSocketNonBlocking(fInputGS->socketNum());
 }
 
 BasicUDPSource::~BasicUDPSource(){
@@ -37,13 +43,17 @@ BasicUDPSource::~BasicUDPSource(){
 }
 
 void BasicUDPSource::doGetNextFrame() {
-  // Await the next incoming packet:
-  envir().taskScheduler().turnOnBackgroundReadHandling(fInputGS->socketNum(), 
-       (TaskScheduler::BackgroundHandlerProc*)&incomingPacketHandler, this);
+  if (!fHaveStartedReading) {
+    // Await incoming packets:
+    envir().taskScheduler().turnOnBackgroundReadHandling(fInputGS->socketNum(),
+	 (TaskScheduler::BackgroundHandlerProc*)&incomingPacketHandler, this);
+    fHaveStartedReading = True;
+  }
 }
 
 void BasicUDPSource::doStopGettingFrames() {
   envir().taskScheduler().turnOffBackgroundReadHandling(fInputGS->socketNum());
+  fHaveStartedReading = False;
 }
 
 
@@ -57,7 +67,7 @@ void BasicUDPSource::incomingPacketHandler1() {
   // Read the packet into our desired destination:
   struct sockaddr_in fromAddress;
   if (!fInputGS->handleRead(fTo, fMaxSize, fFrameSize, fromAddress)) return;
-    
+
   // Tell our client that we have new data:
   afterGetting(this); // we're preceded by a net read; no infinite recursion
 }
