@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2010 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A file source that is a plain byte stream (rather than frames)
 // Implementation
 
@@ -45,10 +45,8 @@ ByteStreamFileSource::createNew(UsageEnvironment& env, char const* fileName,
   FILE* fid = OpenInputFile(env, fileName);
   if (fid == NULL) return NULL;
 
-  Boolean deleteFidOnClose = fid == stdin ? False : True;
   ByteStreamFileSource* newSource
-    = new ByteStreamFileSource(env, fid, deleteFidOnClose,
-			       preferredFrameSize, playTimePerFrame);
+    = new ByteStreamFileSource(env, fid, preferredFrameSize, playTimePerFrame);
   newSource->fFileSize = GetFileSize(fileName, fid);
 
   return newSource;
@@ -56,21 +54,21 @@ ByteStreamFileSource::createNew(UsageEnvironment& env, char const* fileName,
 
 ByteStreamFileSource*
 ByteStreamFileSource::createNew(UsageEnvironment& env, FILE* fid,
-				Boolean deleteFidOnClose,
 				unsigned preferredFrameSize,
 				unsigned playTimePerFrame) {
   if (fid == NULL) return NULL;
 
-  ByteStreamFileSource* newSource
-    = new ByteStreamFileSource(env, fid, deleteFidOnClose,
-			       preferredFrameSize, playTimePerFrame);
+  ByteStreamFileSource* newSource = new ByteStreamFileSource(env, fid, preferredFrameSize, playTimePerFrame);
   newSource->fFileSize = GetFileSize(NULL, fid);
 
   return newSource;
 }
 
-void ByteStreamFileSource::seekToByteAbsolute(u_int64_t byteNumber) {
+void ByteStreamFileSource::seekToByteAbsolute(u_int64_t byteNumber, u_int64_t numBytesToStream) {
   SeekFile64(fFid, (int64_t)byteNumber, SEEK_SET);
+
+  fNumBytesToStream = numBytesToStream;
+  fLimitNumBytesToStream = fNumBytesToStream > 0;
 }
 
 void ByteStreamFileSource::seekToByteRelative(int64_t offset) {
@@ -78,12 +76,22 @@ void ByteStreamFileSource::seekToByteRelative(int64_t offset) {
 }
 
 ByteStreamFileSource::ByteStreamFileSource(UsageEnvironment& env, FILE* fid,
-					   Boolean deleteFidOnClose,
 					   unsigned preferredFrameSize,
 					   unsigned playTimePerFrame)
-  : FramedFileSource(env, fid), fPreferredFrameSize(preferredFrameSize),
-    fPlayTimePerFrame(playTimePerFrame), fLastPlayTime(0), fFileSize(0),
-    fDeleteFidOnClose(deleteFidOnClose), fHaveStartedReading(False) {
+  : FramedFileSource(env, fid), fFileSize(0), fPreferredFrameSize(preferredFrameSize),
+    fPlayTimePerFrame(playTimePerFrame), fLastPlayTime(0),
+    fHaveStartedReading(False), fLimitNumBytesToStream(False), fNumBytesToStream(0) {
+#ifndef READ_FROM_FILES_SYNCHRONOUSLY
+  makeSocketNonBlocking(fileno(fFid));
+#endif
+
+  // Test whether the file is seekable
+  if (SeekFile64(fFid, 1, SEEK_CUR) >= 0) {
+    fFidIsSeekable = True;
+    SeekFile64(fFid, -1, SEEK_CUR);
+  } else {
+    fFidIsSeekable = False;
+  }
 }
 
 ByteStreamFileSource::~ByteStreamFileSource() {
@@ -93,11 +101,11 @@ ByteStreamFileSource::~ByteStreamFileSource() {
   envir().taskScheduler().turnOffBackgroundReadHandling(fileno(fFid));
 #endif
 
-  if (fDeleteFidOnClose) fclose(fFid);
+  CloseInputFile(fFid);
 }
 
 void ByteStreamFileSource::doGetNextFrame() {
-  if (feof(fFid) || ferror(fFid)) {
+  if (feof(fFid) || ferror(fFid) || (fLimitNumBytesToStream && fNumBytesToStream == 0)) {
     handleClosure(this);
     return;
   }
@@ -130,16 +138,28 @@ void ByteStreamFileSource::fileReadableHandler(ByteStreamFileSource* source, int
 }
 
 void ByteStreamFileSource::doReadFromFile() {
-  // Try to read as many bytes as will fit in the buffer provided
-  // (or "fPreferredFrameSize" if less)
+  // Try to read as many bytes as will fit in the buffer provided (or "fPreferredFrameSize" if less)
+  if (fLimitNumBytesToStream && fNumBytesToStream < (u_int64_t)fMaxSize) {
+    fMaxSize = (unsigned)fNumBytesToStream;
+  }
   if (fPreferredFrameSize > 0 && fPreferredFrameSize < fMaxSize) {
     fMaxSize = fPreferredFrameSize;
   }
+#ifdef READ_FROM_FILES_SYNCHRONOUSLY
   fFrameSize = fread(fTo, 1, fMaxSize, fFid);
+#else
+  if (fFidIsSeekable) {
+    fFrameSize = fread(fTo, 1, fMaxSize, fFid);
+  } else {
+    // For non-seekable files (e.g., pipes), call "read()" rather than "fread()", to ensure that the read doesn't block:
+    fFrameSize = read(fileno(fFid), fTo, fMaxSize);
+  }
+#endif
   if (fFrameSize == 0) {
     handleClosure(this);
     return;
   }
+  fNumBytesToStream -= fFrameSize;
 
   // Set the 'presentation time':
   if (fPlayTimePerFrame > 0 && fPreferredFrameSize > 0) {
