@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // MPEG-1 or MPEG-2 Audio RTP Sources
 // Implementation
 
@@ -54,7 +54,7 @@ Boolean MPEG1or2AudioRTPSource
 
   resultSpecialHeaderSize = 4;
   return True;
-}    
+}
 
 char const* MPEG1or2AudioRTPSource::MIMEtype() const {
   return "audio/MPEG";
