@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A filter that produces a sequence of I-frame indices from a MPEG-2 Transport Stream
 // Implementation
 
@@ -24,10 +24,16 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 
 enum RecordType {
   RECORD_UNPARSED = 0,
-  RECORD_VSH = 1,
+  RECORD_VSH = 1, // a MPEG Video Sequence Header
   RECORD_GOP = 2,
   RECORD_PIC_NON_IFRAME = 3, // includes slices
   RECORD_PIC_IFRAME = 4, // includes slices
+  RECORD_NAL_SPS = 5, // H.264	
+  RECORD_NAL_PPS = 6, // H.264	
+  RECORD_NAL_SEI = 7, // H.264	
+  RECORD_NAL_NON_IFRAME = 8, // H.264	
+  RECORD_NAL_IFRAME = 9, // H.264	
+  RECORD_NAL_OTHER = 10, // H.264	
   RECORD_JUNK
 };
 
@@ -36,23 +42,23 @@ public:
   IndexRecord(u_int8_t startOffset, u_int8_t size,
 	      unsigned long transportPacketNumber, float pcr);
   virtual ~IndexRecord();
-  
+
   RecordType& recordType() { return fRecordType; }
   void setFirstFlag() { fRecordType = (RecordType)(((u_int8_t)fRecordType) | 0x80); }
   u_int8_t startOffset() const { return fStartOffset; }
   u_int8_t& size() { return fSize; }
   float pcr() const { return fPCR; }
   unsigned long transportPacketNumber() const { return fTransportPacketNumber; }
-  
+
   IndexRecord* next() const { return fNext; }
   void addAfter(IndexRecord* prev);
   void unlink();
-  
+
 private:
   // Index records are maintained in a doubly-linked list:
   IndexRecord* fNext;
   IndexRecord* fPrev;
-  
+
   RecordType fRecordType;
   u_int8_t fStartOffset; // within the Transport Stream packet
   u_int8_t fSize; // in bytes, following "fStartOffset".
@@ -68,6 +74,12 @@ static char const* recordTypeStr[] = {
   "GOP",
   "PIC(non-I-frame)",
   "PIC(I-frame)",
+  "SPS (H.264)",
+  "PPS (H.264)",
+  "SEI (H.264)",
+  "H.264 non-I-frame",
+  "H.264 I-frame",
+  "other NAL unit (H.264)",
   "JUNK"
 };
 UsageEnvironment& operator<<(UsageEnvironment& env, IndexRecord& r) {
@@ -75,7 +87,7 @@ UsageEnvironment& operator<<(UsageEnvironment& env, IndexRecord& r) {
 	     << recordTypeStr[r.recordType()&0x7F] << ":"
 	     << (unsigned)r.transportPacketNumber() << ":" << r.startOffset()
 	     << "(" << r.size() << ")@" << r.pcr() << "]";
-} 
+}
 #endif
 
 
@@ -101,7 +113,7 @@ MPEG2IFrameIndexFromTransportStream
 ::MPEG2IFrameIndexFromTransportStream(UsageEnvironment& env,
 				      FramedSource* inputSource)
   : FramedFilter(env, inputSource),
-    fInputTransportPacketCounter((unsigned)-1), fClosureNumber(0),
+    fIsH264(False), fInputTransportPacketCounter((unsigned)-1), fClosureNumber(0),
     fLastContinuityCounter(~0),
     fFirstPCR(0.0), fLastPCR(0.0), fHaveSeenFirstPCR(False),
     fPMT_PID(0x10), fVideo_PID(0xE0), // default values
@@ -120,13 +132,13 @@ void MPEG2IFrameIndexFromTransportStream::doGetNextFrame() {
   // Begin by trying to deliver an index record (for an already-parsed frame)
   // to the client:
   if (deliverIndexRecord()) return;
-  
+
   // No more index records are left to deliver, so try to parse a new frame:
   if (parseFrame()) { // success - try again
     doGetNextFrame();
     return;
   }
-  
+
   // We need to read some more Transport Stream packets.  Check whether we have room:
   if (fParseBufferSize - fParseBufferDataEnd < TRANSPORT_PACKET_SIZE) {
     // There's no room left.  Compact the buffer, and check again:
@@ -138,7 +150,7 @@ void MPEG2IFrameIndexFromTransportStream::doGetNextFrame() {
       return;
     }
   }
-  
+
   // Arrange to read a new Transport Stream packet:
   fInputSource->getNextFrame(fInputBuffer, sizeof fInputBuffer,
 			     afterGettingFrame, this,
@@ -156,27 +168,29 @@ void MPEG2IFrameIndexFromTransportStream
 			     presentationTime, durationInMicroseconds);
 }
 
+#define TRANSPORT_SYNC_BYTE 0x47
+
 void MPEG2IFrameIndexFromTransportStream
 ::afterGettingFrame1(unsigned frameSize,
 		     unsigned numTruncatedBytes,
 		     struct timeval presentationTime,
 		     unsigned durationInMicroseconds) {
-  if (frameSize < TRANSPORT_PACKET_SIZE || fInputBuffer[0] != 0x47/*sync byte*/) {
-    if (fInputBuffer[0] != 0x47) {
+  if (frameSize < TRANSPORT_PACKET_SIZE || fInputBuffer[0] != TRANSPORT_SYNC_BYTE) {
+    if (fInputBuffer[0] != TRANSPORT_SYNC_BYTE) {
       envir() << "Bad TS sync byte: 0x" << fInputBuffer[0] << "\n";
     }
     // Handle this as if the source ended:
     handleInputClosure1();
     return;
   }
-  
+
   ++fInputTransportPacketCounter;
-  
+
   // Figure out how much of this Transport Packet contains PES data:
   u_int8_t adaptation_field_control = (fInputBuffer[3]&0x30)>>4;
   u_int8_t totalHeaderSize
     = adaptation_field_control == 1 ? 4 : 5 + fInputBuffer[4];
-  
+
   // Check for a PCR:
   if (totalHeaderSize > 5 && (fInputBuffer[5]&0x10) != 0) {
     // There's a PCR:
@@ -187,14 +201,20 @@ void MPEG2IFrameIndexFromTransportStream
     if ((fInputBuffer[10]&0x80) != 0) pcr += 1/90000.0f; // add in low-bit (if set)
     unsigned short pcrExt = ((fInputBuffer[10]&0x01)<<8) | fInputBuffer[11];
     pcr += pcrExt/27000000.0f;
-    
+
     if (!fHaveSeenFirstPCR) {
       fFirstPCR = pcr;
       fHaveSeenFirstPCR = True;
+    } else if (pcr < fLastPCR) {
+      // The PCR timestamp has gone backwards.  DIsplay a warning about this (because it indicates buggy Transport Stream data),
+      // and compensate for it.
+      envir() << "\nWarning: At about " << fLastPCR-fFirstPCR << " seconds into the file, the PCR timestamp decreased - from "
+	      << fLastPCR << " to " << pcr << "\n";
+      fFirstPCR -= (fLastPCR - pcr);
     }
     fLastPCR = pcr;
   }
-  
+
   // Get the PID from the packet, and check for special tables: the PAT and PMT:
   u_int16_t PID = ((fInputBuffer[1]&0x1F)<<8) | fInputBuffer[2];
   if (PID == PAT_PID) {
@@ -213,7 +233,7 @@ void MPEG2IFrameIndexFromTransportStream
     return;
   }
   fLastContinuityCounter = continuity_counter;
-  
+
   // Also, if this is the start of a PES packet, then skip over the PES header:
   Boolean payload_unit_start_indicator = (fInputBuffer[1]&0x40) != 0;
   //fprintf(stderr, "PUSI: %d\n", payload_unit_start_indicator);//#####
@@ -229,16 +249,16 @@ void MPEG2IFrameIndexFromTransportStream
       return;
     }
   }
-  
+
   // The remaining data is Video Elementary Stream data.  Add it to our parse buffer:
   unsigned vesSize = TRANSPORT_PACKET_SIZE - totalHeaderSize;
   memmove(&fParseBuffer[fParseBufferDataEnd], &fInputBuffer[totalHeaderSize], vesSize);
   fParseBufferDataEnd += vesSize;
-  
+
   // And add a new index record noting where it came from:
   addToTail(new IndexRecord(totalHeaderSize, vesSize, fInputTransportPacketCounter,
 			    fLastPCR - fFirstPCR));
-  
+
   // Try again:
   doGetNextFrame();
 }
@@ -266,7 +286,7 @@ void MPEG2IFrameIndexFromTransportStream::handleInputClosure1() {
     fParseBuffer[fParseBufferDataEnd++] = 0;
     fParseBuffer[fParseBufferDataEnd++] = 1;
     fParseBuffer[fParseBufferDataEnd++] = PICTURE_START_CODE;
-    
+
     // Try again:
     doGetNextFrame();
   } else {
@@ -277,11 +297,15 @@ void MPEG2IFrameIndexFromTransportStream::handleInputClosure1() {
 
 void MPEG2IFrameIndexFromTransportStream
 ::analyzePAT(unsigned char* pkt, unsigned size) {
-  // Get the PMT_PID (we assume that's there just 1 program):
-  if (size < 16) return; // table too small 
-  u_int16_t program_number = (pkt[9]<<8) | pkt[10];
-  if (program_number != 0) {
-    fPMT_PID = ((pkt[11]&0x1F)<<8) | pkt[12];
+  // Get the PMT_PID:
+  while (size >= 17) { // The table is large enough
+    u_int16_t program_number = (pkt[9]<<8) | pkt[10];
+    if (program_number != 0) {
+      fPMT_PID = ((pkt[11]&0x1F)<<8) | pkt[12];
+      return;
+    }
+
+    pkt += 4; size -= 4;
   }
 }
 
@@ -305,8 +329,9 @@ void MPEG2IFrameIndexFromTransportStream
   while (size >= 9) {
     u_int8_t stream_type = pkt[0];
     u_int16_t elementary_PID = ((pkt[1]&0x1F)<<8) | pkt[2];
-    if (stream_type == 1 || stream_type == 2) {
-      fVideo_PID = elementary_PID;
+    if (stream_type == 1 || stream_type == 2 || stream_type == 0x1B/*H.264 video*/) {
+	if (stream_type == 0x1B) fIsH264 = True;
+	fVideo_PID = elementary_PID;
       return;
     }
 
@@ -320,10 +345,10 @@ void MPEG2IFrameIndexFromTransportStream
 Boolean MPEG2IFrameIndexFromTransportStream::deliverIndexRecord() {
   IndexRecord* head = fHeadIndexRecord;
   if (head == NULL) return False;
-  
+
   // Check whether the head record has been parsed yet:
   if (head->recordType() == RECORD_UNPARSED) return False;
-  
+
   // Remove the head record (the one whose data we'll be delivering):
   IndexRecord* next = head->next();
   head->unlink();
@@ -332,13 +357,14 @@ Boolean MPEG2IFrameIndexFromTransportStream::deliverIndexRecord() {
   } else {
     fHeadIndexRecord = next;
   }
-  
+
   if (head->recordType() == RECORD_JUNK) {
     // Don't actually deliver the data to the client:
     delete head;
-    return True;
+    // Try to deliver the next record instead:
+    return deliverIndexRecord();
   }
-  
+
   // Deliver data from the head record:
 #ifdef DEBUG
   envir() << "delivering: " << *head << "\n";
@@ -365,10 +391,10 @@ Boolean MPEG2IFrameIndexFromTransportStream::deliverIndexRecord() {
     fTo[10] = (unsigned char)(tpn>>24);
     fFrameSize = 11;
   }
-  
+
   // Free the (former) head record (as we're now done with it):
   delete head;
-  
+
   // Complete delivery to the client:
   afterGetting(this);
   return True;
@@ -378,9 +404,10 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
   // At this point, we have a queue of >=0 (unparsed) index records, representing
   // the data in the parse buffer from "fParseBufferFrameStart"
   // to "fParseBufferDataEnd".  We now parse through this data, looking for
-  // a complete 'frame' (where a 'frame', in this case, means
-  // a Video Sequence Header, GOP Header, Picture Header, or Slice).
-  
+  // a complete 'frame', where a 'frame', in this case, means:
+  // 	for MPEG video: a Video Sequence Header, GOP Header, Picture Header, or Slice
+  // 	for H.264 video: a NAL unit
+
   // Inspect the frame's initial 4-byte code, to make sure it starts with a system code:
   if (fParseBufferDataEnd-fParseBufferFrameStart < 4) return False; // not enough data
   unsigned numInitialBadBytes = 0;
@@ -393,15 +420,16 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
     }
     unsigned char nextCode;
     if (!parseToNextCode(nextCode)) return False;
-    
+
     numInitialBadBytes = fParseBufferParseEnd - fParseBufferFrameStart;
-    //fprintf(stderr, "#####numInitialBadBytes: 0x%x\n", numInitialBadBytes);
+    //fprintf(stderr, "#####numInitialBadBytes: %d (0x%x)\n", numInitialBadBytes, numInitialBadBytes);
     fParseBufferFrameStart = fParseBufferParseEnd;
     fParseBufferParseEnd += 4; // skip over the code that we just saw
     p = &fParseBuffer[fParseBufferFrameStart];
   }
-  
+
   unsigned char curCode = p[3];
+  if (fIsH264) curCode &= 0x1F; // nal_unit_type
   RecordType curRecordType;
   unsigned char nextCode;
   switch (curCode) {
@@ -426,19 +454,44 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
     }
     break;
   }
-  default: { // picture (including slices)
-    curRecordType = RECORD_PIC_NON_IFRAME; // may get changed to IFRAME later
-    while (1) {
+  case 1: // Coded slice of a non-IDR picture (H.264)
+    curRecordType = RECORD_NAL_NON_IFRAME;
+    if (!parseToNextCode(nextCode)) return False;
+    break;
+  case 5: // Coded slice of an IDR picture (H.264) 
+    curRecordType = RECORD_NAL_IFRAME;
+    if (!parseToNextCode(nextCode)) return False;
+    break;
+  case 6: // Supplemental enhancement information (SEI) (H.264)
+    curRecordType = RECORD_NAL_SEI;
+    if (!parseToNextCode(nextCode)) return False;
+    break;
+  case 7: // Sequence parameter set (SPS) (H.264)
+    curRecordType = RECORD_NAL_SPS;
+    if (!parseToNextCode(nextCode)) return False;
+    break;
+  case 8: // Picture parameter set (PPS) (H.264)
+    curRecordType = RECORD_NAL_PPS;
+    if (!parseToNextCode(nextCode)) return False;
+    break;
+  default: { // picture (including slices), or some other H.264 NAL unit
+    if (fIsH264) {
+      curRecordType = RECORD_NAL_OTHER;
       if (!parseToNextCode(nextCode)) return False;
-      if (nextCode == VIDEO_SEQUENCE_START_CODE || nextCode == VISUAL_OBJECT_SEQUENCE_START_CODE ||
-	  nextCode == GROUP_START_CODE || nextCode == GROUP_VOP_START_CODE ||
-	  nextCode == PICTURE_START_CODE || nextCode == VOP_START_CODE) break;
-      fParseBufferParseEnd += 4; // skip over the code that we just saw
+    } else {
+      curRecordType = RECORD_PIC_NON_IFRAME; // may get changed to IFRAME later
+      while (1) {
+        if (!parseToNextCode(nextCode)) return False;
+        if (nextCode == VIDEO_SEQUENCE_START_CODE || nextCode == VISUAL_OBJECT_SEQUENCE_START_CODE ||
+	    nextCode == GROUP_START_CODE || nextCode == GROUP_VOP_START_CODE ||
+	    nextCode == PICTURE_START_CODE || nextCode == VOP_START_CODE) break;
+        fParseBufferParseEnd += 4; // skip over the code that we just saw
+      }
     }
     break;
   }
   }
-  
+
   if (curRecordType == RECORD_PIC_NON_IFRAME) {
     if (curCode == VOP_START_CODE) { // MPEG-4
       //fprintf(stderr, "#####parseFrame()1(4): 0x%x, 0x%x\n", curCode, fParseBuffer[fParseBufferFrameStart+4]&0xC0);
@@ -454,7 +507,7 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
       }
     }
   }
-  
+
   // There is now a parsed 'frame', from "fParseBufferFrameStart"
   // to "fParseBufferParseEnd". Tag the corresponding index records to note this:
   unsigned frameSize = fParseBufferParseEnd - fParseBufferFrameStart + numInitialBadBytes;
@@ -471,7 +524,7 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
     }
     if (r == fHeadIndexRecord) r->setFirstFlag();
     // indicates that this is the first record for this frame
-    
+
     if (r->size() > frameSize) {
       // This record contains extra data that's not part of the frame.
       // Shorten this record, and move the extra data to a new record
@@ -482,7 +535,7 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
 #ifdef DEBUG
       envir() << "tagged record (modified): " << *r << "\n";
 #endif
-      
+
       IndexRecord* newRecord
 	= new IndexRecord(newOffset, newSize, r->transportPacketNumber(), r->pcr());
       newRecord->addAfter(r);
@@ -501,12 +554,12 @@ Boolean MPEG2IFrameIndexFromTransportStream::parseFrame() {
       envir() << "!!!!!Internal consistency error!!!!!\n";
       return False;
     }
-  }      
-  
+  }
+
   // Finally, update our parse state (to skip over the now-parsed data):
   fParseBufferFrameStart = fParseBufferParseEnd;
   fParseBufferParseEnd += 4; // to skip over the next code (that we found)
-  
+
   return True;
 }
 
@@ -524,7 +577,7 @@ Boolean MPEG2IFrameIndexFromTransportStream
       return True;
     } else p += 3;
   }
-  
+
   fParseBufferParseEnd = p - &fParseBuffer[0]; // where we've gotten to
   return False; // no luck this time
 }
@@ -532,7 +585,7 @@ Boolean MPEG2IFrameIndexFromTransportStream
 void MPEG2IFrameIndexFromTransportStream::compactParseBuffer() {
 #ifdef DEBUG
   envir() << "Compacting parse buffer: [" << fParseBufferFrameStart
-	  << "," << fParseBufferParseEnd << "," << fParseBufferDataEnd << "]"; 
+	  << "," << fParseBufferParseEnd << "," << fParseBufferDataEnd << "]";
 #endif
   memmove(&fParseBuffer[0], &fParseBuffer[fParseBufferFrameStart],
 	  fParseBufferDataEnd - fParseBufferFrameStart);
@@ -541,7 +594,7 @@ void MPEG2IFrameIndexFromTransportStream::compactParseBuffer() {
   fParseBufferFrameStart = 0;
 #ifdef DEBUG
   envir() << "-> [" << fParseBufferFrameStart
-	  << "," << fParseBufferParseEnd << "," << fParseBufferDataEnd << "]\n"; 
+	  << "," << fParseBufferParseEnd << "," << fParseBufferDataEnd << "]\n";
 #endif
 }
 
