@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // Media
 // Implementation
 
@@ -33,7 +33,7 @@ public:
   void remove(char const* name);
 
   void generateNewName(char* mediumName, unsigned maxLen);
-  
+
 protected:
   MediaLookupTable(UsageEnvironment& env);
   virtual ~MediaLookupTable();
@@ -118,8 +118,8 @@ Boolean Medium::isDarwinInjector() const {
 
 ////////// _Tables implementation //////////
 
-_Tables* _Tables::getOurTables(UsageEnvironment& env) {
-  if (env.liveMediaPriv == NULL) {
+_Tables* _Tables::getOurTables(UsageEnvironment& env, Boolean createIfNotPresent) {
+  if (env.liveMediaPriv == NULL && createIfNotPresent) {
     env.liveMediaPriv = new _Tables(env);
   }
   return (_Tables*)(env.liveMediaPriv);
