@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // RTCP
 // Implementation
 
@@ -142,13 +142,14 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
   }
 
   if (isSSMSource) RTCPgs->multicastSendOnly(); // don't receive multicast
-    
+
   double timeNow = dTimeNow();
   fPrevReportTime = fNextReportTime = timeNow;
 
   fKnownMembers = new RTCPMemberDatabase(*this);
   fInBuf = new unsigned char[maxPacketSize];
   if (fKnownMembers == NULL || fInBuf == NULL) return;
+  fNumBytesAlreadyRead = 0;
 
   // A hack to save buffer space, because RTCP packets are always small:
   unsigned savedMaxSize = OutPacketBuffer::maxSize;
@@ -161,7 +162,7 @@ RTCPInstance::RTCPInstance(UsageEnvironment& env, Groupsock* RTCPgs,
   TaskScheduler::BackgroundHandlerProc* handler
     = (TaskScheduler::BackgroundHandlerProc*)&incomingReportHandler;
   fRTCPInterface.startNetworkReading(handler);
-  
+
   // Send our first report.
   fTypeOfEvent = EVENT_REPORT;
   onExpire(this);
@@ -296,8 +297,11 @@ void RTCPInstance::setStreamSocket(int sockNum,
 
 void RTCPInstance::addStreamSocket(int sockNum,
 				   unsigned char streamChannelId) {
+  // First, turn off background read handling for the default (UDP) socket:
+  envir().taskScheduler().turnOffBackgroundReadHandling(fRTCPInterface.gs()->socketNum());
+
   // Add the RTCP-over-TCP interface:
-  fRTCPInterface.setStreamSocket(sockNum, streamChannelId);
+  fRTCPInterface.addStreamSocket(sockNum, streamChannelId);
 
   // Turn on background reading for this socket (in case it's not on already):
   TaskScheduler::BackgroundHandlerProc* handler
@@ -316,21 +320,29 @@ void RTCPInstance::incomingReportHandler(RTCPInstance* instance,
 }
 
 void RTCPInstance::incomingReportHandler1() {
-  unsigned char* pkt = fInBuf;
-  unsigned packetSize;
-  struct sockaddr_in fromAddress;
-  int typeOfPacket = PACKET_UNKNOWN_TYPE;
-
   do {
     int tcpReadStreamSocketNum = fRTCPInterface.nextTCPReadStreamSocketNum();
     unsigned char tcpReadStreamChannelId = fRTCPInterface.nextTCPReadStreamChannelId();
-    if (!fRTCPInterface.handleRead(pkt, maxPacketSize,
-				   packetSize, fromAddress)) {
-      break;
+    unsigned packetSize = 0;
+    unsigned numBytesRead;
+    struct sockaddr_in fromAddress;
+    Boolean packetReadWasIncomplete;
+    Boolean readResult
+      = fRTCPInterface.handleRead(&fInBuf[fNumBytesAlreadyRead], maxPacketSize - fNumBytesAlreadyRead,
+				  numBytesRead, fromAddress, packetReadWasIncomplete);
+    if (packetReadWasIncomplete) {
+      fNumBytesAlreadyRead += numBytesRead;
+      return; // more reads are needed to get the entire packet
+    } else { // normal case: We've read the entire packet 
+      packetSize = fNumBytesAlreadyRead + numBytesRead;
+      fNumBytesAlreadyRead = 0; // for next time
     }
+    if (!readResult) break;
 
     // Ignore the packet if it was looped-back from ourself:
+    Boolean packetWasFromOurHost = False;
     if (RTCPgs()->wasLoopedBackFromUs(envir(), fromAddress)) {
+      packetWasFromOurHost = True;
       // However, we still want to handle incoming RTCP packets from
       // *other processes* on the same machine.  To distinguish this
       // case from a true loop-back, check whether we've just sent a
@@ -343,13 +355,23 @@ void RTCPInstance::incomingReportHandler1() {
       }
     }
 
-    if (fIsSSMSource) {
-      // This packet was received via unicast.  'Reflect' it by resending
-      // it to the multicast group.
+    unsigned char* pkt = fInBuf;
+    if (fIsSSMSource && !packetWasFromOurHost) {
+      // This packet is assumed to have been received via unicast (because we're a SSM source, and SSM receivers send back RTCP "RR"
+      // packets via unicast).  'Reflect' the packet by resending it to the multicast group, so that any other receivers can also
+      // get to see it.
+
       // NOTE: Denial-of-service attacks are possible here.
       // Users of this software may wish to add their own,
       // application-specific mechanism for 'authenticating' the
-      // validity of this packet before relecting it.
+      // validity of this packet before reflecting it.
+
+      // NOTE: The test for "!packetWasFromOurHost" means that we won't reflect RTCP packets that come from other processes on
+      // the same host as us.  The reason for this is that the 'packet size' test above is not 100% reliable; some packets
+      // that were truly looped back from us might not be detected as such, and this might lead to infinite forwarding/receiving
+      // of some packets.  To avoid this possibility, we only reflect RTCP packets that we know for sure originated elsewhere.
+      // (Note, though, that if we ever re-enable the code in "Groupsock::multicastSendOnly()", then we could remove the test for
+      // "!packetWasFromOurHost".)
       fRTCPInterface.sendPacket(pkt, packetSize);
       fHaveJustSentPacket = True;
       fLastPacketSentSize = packetSize;
@@ -357,10 +379,9 @@ void RTCPInstance::incomingReportHandler1() {
 
 #ifdef DEBUG
     fprintf(stderr, "[%p]saw incoming RTCP packet (from address %s, port %d)\n", this, our_inet_ntoa(fromAddress.sin_addr), ntohs(fromAddress.sin_port));
-    unsigned char* p = pkt;
     for (unsigned i = 0; i < packetSize; ++i) {
       if (i%4 == 0) fprintf(stderr, " ");
-      fprintf(stderr, "%02x", p[i]);
+      fprintf(stderr, "%02x", pkt[i]);
     }
     fprintf(stderr, "\n");
 #endif
@@ -371,7 +392,7 @@ void RTCPInstance::incomingReportHandler1() {
     // must be version=2, with no padding bit, and a payload type of
     // SR (200) or RR (201):
     if (packetSize < 4) break;
-    unsigned rtcpHdr = ntohl(*(unsigned*)pkt);
+    unsigned rtcpHdr = ntohl(*(u_int32_t*)pkt);
     if ((rtcpHdr & 0xE0FE0000) != (0x80000000 | (RTCP_PT_SR<<16))) {
 #ifdef DEBUG
       fprintf(stderr, "rejected bad RTCP packet: header 0x%08x\n", rtcpHdr);
@@ -381,6 +402,7 @@ void RTCPInstance::incomingReportHandler1() {
 
     // Process each of the individual RTCP 'subpackets' in (what may be)
     // a compound RTCP packet.
+    int typeOfPacket = PACKET_UNKNOWN_TYPE;
     unsigned reportSenderSSRC = 0;
     Boolean packetOK = False;
     while (1) {
@@ -392,7 +414,7 @@ void RTCPInstance::incomingReportHandler1() {
 
       // Assume that each RTCP subpacket begins with a 4-byte SSRC:
       if (length < 4) break; length -= 4;
-      reportSenderSSRC = ntohl(*(unsigned*)pkt); ADVANCE(4);
+      reportSenderSSRC = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
 
       Boolean subPacketOK = False;
       switch (pt) {
@@ -403,9 +425,9 @@ void RTCPInstance::incomingReportHandler1() {
 	  if (length < 20) break; length -= 20;
 
 	  // Extract the NTP timestamp, and note this:
-	  unsigned NTPmsw = ntohl(*(unsigned*)pkt); ADVANCE(4);
-	  unsigned NTPlsw = ntohl(*(unsigned*)pkt); ADVANCE(4);
-	  unsigned rtpTimestamp = ntohl(*(unsigned*)pkt); ADVANCE(4);
+	  unsigned NTPmsw = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+	  unsigned NTPlsw = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+	  unsigned rtpTimestamp = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
 	  if (fSource != NULL) {
 	    RTPReceptionStatsDB& receptionStats
 	      = fSource->receptionStatsDB();
@@ -431,14 +453,14 @@ void RTCPInstance::incomingReportHandler1() {
 	    // Use this information to update stats about our transmissions:
             RTPTransmissionStatsDB& transmissionStats = fSink->transmissionStatsDB();
             for (unsigned i = 0; i < rc; ++i) {
-              unsigned senderSSRC = ntohl(*(unsigned*)pkt); ADVANCE(4);
+              unsigned senderSSRC = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
               // We care only about reports about our own transmission, not others'
               if (senderSSRC == fSink->SSRC()) {
-                unsigned lossStats = ntohl(*(unsigned*)pkt); ADVANCE(4);
-                unsigned highestReceived = ntohl(*(unsigned*)pkt); ADVANCE(4);
-                unsigned jitter = ntohl(*(unsigned*)pkt); ADVANCE(4);
-                unsigned timeLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
-                unsigned timeSinceLastSR = ntohl(*(unsigned*)pkt); ADVANCE(4);
+                unsigned lossStats = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+                unsigned highestReceived = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+                unsigned jitter = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+                unsigned timeLastSR = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
+                unsigned timeSinceLastSR = ntohl(*(u_int32_t*)pkt); ADVANCE(4);
                 transmissionStats.noteIncomingRR(reportSenderSSRC, fromAddress,
 						 lossStats,
 						 highestReceived, jitter,
@@ -468,7 +490,7 @@ void RTCPInstance::incomingReportHandler1() {
 		fromAddr = tcpReadStreamSocketNum;
 		fromPortNum = tcpReadStreamChannelId;
 	      }
-	      Port fromPort(fromPortNum); 
+	      Port fromPort(fromPortNum);
 	      RRHandlerRecord* rrHandler
 		= (RRHandlerRecord*)(fSpecificRRHandlerTable->Lookup(fromAddr, (~0), fromPort));
 	      if (rrHandler != NULL) {
@@ -499,7 +521,7 @@ void RTCPInstance::incomingReportHandler1() {
 		  || (fSink != NULL
 		      && fSink->transmissionStatsDB().lookup(reportSenderSSRC) != NULL))) {
 	    fByeHandlerTask = NULL;
-	        // we call this only once by default 
+	        // we call this only once by default
 	    (*byeHandler)(fByeHandlerClientData);
 	  }
 
@@ -516,7 +538,7 @@ void RTCPInstance::incomingReportHandler1() {
 #endif
 	  subPacketOK = True;
 	  break;
-      }  
+      }
       if (!subPacketOK) break;
 
       // need to check for (& handle) SSRC collision! #####
@@ -524,7 +546,7 @@ void RTCPInstance::incomingReportHandler1() {
 #ifdef DEBUG
       fprintf(stderr, "validated RTCP subpacket (type %d): %d, %d, %d, 0x%08x\n", typeOfPacket, rc, pt, length, reportSenderSSRC);
 #endif
-      
+
       // Skip over any remaining bytes in this subpacket:
       ADVANCE(length);
 
@@ -538,7 +560,7 @@ void RTCPInstance::incomingReportHandler1() {
 #endif
 	break;
       }
-      rtcpHdr = ntohl(*(unsigned*)pkt);
+      rtcpHdr = ntohl(*(u_int32_t*)pkt);
       if ((rtcpHdr & 0xC0000000) != 0x80000000) {
 #ifdef DEBUG
 	fprintf(stderr, "bad RTCP subpacket: header 0x%08x\n", rtcpHdr);
@@ -546,7 +568,7 @@ void RTCPInstance::incomingReportHandler1() {
 	break;
       }
     }
-      
+
     if (!packetOK) {
 #ifdef DEBUG
       fprintf(stderr, "rejected bad RTCP subpacket: header 0x%08x\n", rtcpHdr);
@@ -557,7 +579,7 @@ void RTCPInstance::incomingReportHandler1() {
       fprintf(stderr, "validated entire RTCP packet\n");
 #endif
     }
-      
+
     onReceive(typeOfPacket, totPacketSize, reportSenderSSRC);
   } while (0);
 }
@@ -733,7 +755,7 @@ void RTCPInstance::enqueueCommonReportPrefix(unsigned char packetType,
 
 void RTCPInstance::enqueueCommonReportSuffix() {
   // Output the report blocks for each source:
-  if (fSource != NULL) { 
+  if (fSource != NULL) {
     RTPReceptionStatsDB& allReceptionStats
       = fSource->receptionStatsDB();
 
@@ -768,7 +790,7 @@ RTCPInstance::enqueueReportBlock(RTPReceptionStats* stats) {
   unsigned numExpectedSinceLastReset
     = highestExtSeqNumReceived - stats->lastResetExtSeqNumReceived();
   int numLostSinceLastReset
-    = numExpectedSinceLastReset - stats->numPacketsReceivedSinceLastReset(); 
+    = numExpectedSinceLastReset - stats->numPacketsReceivedSinceLastReset();
   unsigned char lossFraction;
   if (numExpectedSinceLastReset == 0 || numLostSinceLastReset < 0) {
     lossFraction = 0;
@@ -776,7 +798,7 @@ RTCPInstance::enqueueReportBlock(RTPReceptionStats* stats) {
     lossFraction = (unsigned char)
       ((numLostSinceLastReset << 8) / numExpectedSinceLastReset);
   }
-  
+
   fOutBuf->enqueueWord((lossFraction<<24) | totNumLost);
   fOutBuf->enqueueWord(highestExtSeqNumReceived);
 
@@ -798,7 +820,7 @@ RTCPInstance::enqueueReportBlock(RTPReceptionStats* stats) {
   timeSinceLSR.tv_sec = timeNow.tv_sec - LSRtime.tv_sec;
   timeSinceLSR.tv_usec = timeNow.tv_usec - LSRtime.tv_usec;
   // The enqueued time is in units of 1/65536 seconds.
-  // (Note that 65536/1000000 == 1024/15625) 
+  // (Note that 65536/1000000 == 1024/15625)
   unsigned DLSR;
   if (LSR == 0) {
     DLSR = 0;
@@ -857,10 +879,11 @@ void RTCPInstance::schedule(double nextTime) {
   fNextReportTime = nextTime;
 
   double secondsToDelay = nextTime - dTimeNow();
+  if (secondsToDelay < 0) secondsToDelay = 0;
 #ifdef DEBUG
   fprintf(stderr, "schedule(%f->%f)\n", secondsToDelay, nextTime);
 #endif
-  int usToGo = (int)(secondsToDelay * 1000000);
+  int64_t usToGo = (int64_t)(secondsToDelay * 1000000);
   nextTask() = envir().taskScheduler().scheduleDelayedTask(usToGo,
 				(TaskFunc*)RTCPInstance::onExpire, this);
 }
@@ -891,14 +914,11 @@ void RTCPInstance::onExpire1() {
 
 SDESItem::SDESItem(unsigned char tag, unsigned char const* value) {
   unsigned length = strlen((char const*)value);
-  if (length > 511) length = 511;
+  if (length > 0xFF) length = 0xFF; // maximum data length for a SDES item
 
   fData[0] = tag;
   fData[1] = (unsigned char)length;
   memmove(&fData[2], value, length);
-
-  // Pad the trailing bytes to a 4-byte boundary:
-  while ((length)%4 > 0) fData[2 + length++] = '\0';
 }
 
 unsigned SDESItem::totalSize() const {
