@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // An abstraction of a network interface used for RTP (or RTCP).
 // (This allows the RTP-over-TCP hack (RFC 2326, section 10.12) to
 // be implemented transparently.)
@@ -37,8 +37,10 @@ static void sendRTPOverTCP(unsigned char* packet, unsigned packetSize,
 // "SocketDescriptor" that contains a hash table for each of the
 // sub-channels that are reading from this socket.
 
-static HashTable* socketHashTable(UsageEnvironment& env) {
-  _Tables* ourTables = _Tables::getOurTables(env);
+static HashTable* socketHashTable(UsageEnvironment& env, Boolean createIfNotPresent = True) {
+  _Tables* ourTables = _Tables::getOurTables(env, createIfNotPresent);
+  if (ourTables == NULL) return NULL;
+
   if (ourTables->socketTable == NULL) {
     // Create a new socket number -> SocketDescriptor mapping table:
     ourTables->socketTable = HashTable::create(ONE_WORD_HASH_KEYS);
@@ -56,19 +58,37 @@ public:
   RTPInterface* lookupRTPInterface(unsigned char streamChannelId);
   void deregisterRTPInterface(unsigned char streamChannelId);
 
+  void setServerRequestAlternativeByteHandler(ServerRequestAlternativeByteHandler* handler, void* clientData) {
+    fServerRequestAlternativeByteHandler = handler;
+    fServerRequestAlternativeByteHandlerClientData = clientData;
+  }
+
 private:
   static void tcpReadHandler(SocketDescriptor*, int mask);
+  void tcpReadHandler1(int mask);
 
 private:
   UsageEnvironment& fEnv;
   int fOurSocketNum;
   HashTable* fSubChannelHashTable;
+  ServerRequestAlternativeByteHandler* fServerRequestAlternativeByteHandler;
+  void* fServerRequestAlternativeByteHandlerClientData;
+  u_int8_t fStreamChannelId, fSizeByte1;
+  enum { AWAITING_DOLLAR, AWAITING_STREAM_CHANNEL_ID, AWAITING_SIZE1, AWAITING_SIZE2, AWAITING_PACKET_DATA } fTCPReadingState;
 };
 
-static SocketDescriptor* lookupSocketDescriptor(UsageEnvironment& env,
-						int sockNum) {
+static SocketDescriptor* lookupSocketDescriptor(UsageEnvironment& env, int sockNum, Boolean createIfNotFound = True) {
+  HashTable* table = socketHashTable(env, createIfNotFound);
+  if (table == NULL) return NULL;
+
   char const* key = (char const*)(long)sockNum;
-  return (SocketDescriptor*)(socketHashTable(env)->Lookup(key));
+  SocketDescriptor* socketDescriptor = (SocketDescriptor*)(table->Lookup(key));
+  if (socketDescriptor == NULL && createIfNotFound) {
+    socketDescriptor = new SocketDescriptor(env, sockNum);
+    table->Add((char const*)(long)(sockNum), socketDescriptor);
+  }
+
+  return socketDescriptor;
 }
 
 static void removeSocketDescription(UsageEnvironment& env, int sockNum) {
@@ -94,14 +114,18 @@ RTPInterface::RTPInterface(Medium* owner, Groupsock* gs)
     fNextTCPReadSize(0), fNextTCPReadStreamSocketNum(-1),
     fNextTCPReadStreamChannelId(0xFF), fReadHandlerProc(NULL),
     fAuxReadHandlerFunc(NULL), fAuxReadHandlerClientData(NULL) {
+  // Make the socket non-blocking, even though it will be read from only asynchronously, when packets arrive.
+  // The reason for this is that, in some OSs, reads on a blocking socket can (allegedly) sometimes block,
+  // even if the socket was previously reported (e.g., by "select()") as having data available.
+  // (This can supposedly happen if the UDP checksum fails, for example.)
+  makeSocketNonBlocking(fGS->socketNum());
+  increaseSendBufferTo(envir(), fGS->socketNum(), 50*1024);
 }
 
 RTPInterface::~RTPInterface() {
   delete fTCPStreams;
 }
 
-Boolean RTPOverTCP_OK = True; // HACK: For detecting TCP socket failure externally #####
-
 void RTPInterface::setStreamSocket(int sockNum,
 				   unsigned char streamChannelId) {
   fGS->removeAllDestinations();
@@ -111,7 +135,6 @@ void RTPInterface::setStreamSocket(int sockNum,
 void RTPInterface::addStreamSocket(int sockNum,
 				   unsigned char streamChannelId) {
   if (sockNum < 0) return;
-  else RTPOverTCP_OK = True; //##### HACK
 
   for (tcpStreamRecord* streams = fTCPStreams; streams != NULL;
        streams = streams->fNext) {
@@ -124,13 +147,24 @@ void RTPInterface::addStreamSocket(int sockNum,
   fTCPStreams = new tcpStreamRecord(sockNum, streamChannelId, fTCPStreams);
 }
 
+static void deregisterSocket(UsageEnvironment& env, int sockNum, unsigned char streamChannelId) {
+  SocketDescriptor* socketDescriptor = lookupSocketDescriptor(env, sockNum, False);
+  if (socketDescriptor != NULL) {
+    socketDescriptor->deregisterRTPInterface(streamChannelId);
+        // Note: This may delete "socketDescriptor",
+        // if no more interfaces are using this socket
+  }
+}
+
 void RTPInterface::removeStreamSocket(int sockNum,
 				      unsigned char streamChannelId) {
   for (tcpStreamRecord** streamsPtr = &fTCPStreams; *streamsPtr != NULL;
        streamsPtr = &((*streamsPtr)->fNext)) {
     if ((*streamsPtr)->fStreamSocketNum == sockNum
 	&& (*streamsPtr)->fStreamChannelId == streamChannelId) {
-      // Remove the record pointed to by *streamsPtr :
+      deregisterSocket(envir(), sockNum, streamChannelId);
+
+      // Then remove the record pointed to by *streamsPtr :
       tcpStreamRecord* next = (*streamsPtr)->fNext;
       (*streamsPtr)->fNext = NULL;
       delete (*streamsPtr);
@@ -140,6 +174,14 @@ void RTPInterface::removeStreamSocket(int sockNum,
   }
 }
 
+void RTPInterface
+::setServerRequestAlternativeByteHandler(int socketNum, ServerRequestAlternativeByteHandler* handler, void* clientData) {
+  SocketDescriptor* socketDescriptor = lookupSocketDescriptor(envir(), socketNum);
+
+  if (socketDescriptor != NULL) socketDescriptor->setServerRequestAlternativeByteHandler(handler, clientData);
+}
+
+
 void RTPInterface::sendPacket(unsigned char* packet, unsigned packetSize) {
   // Normal case: Send as a UDP packet:
   fGS->output(envir(), fGS->ttl(), packet, packetSize);
@@ -163,24 +205,16 @@ void RTPInterface
   for (tcpStreamRecord* streams = fTCPStreams; streams != NULL;
        streams = streams->fNext) {
     // Get a socket descriptor for "streams->fStreamSocketNum":
-    SocketDescriptor* socketDescriptor
-      = lookupSocketDescriptor(envir(), streams->fStreamSocketNum);
-    if (socketDescriptor == NULL) {
-      socketDescriptor
-	= new SocketDescriptor(envir(), streams->fStreamSocketNum);
-      socketHashTable(envir())->Add((char const*)(long)(streams->fStreamSocketNum),
-				    socketDescriptor);
-    }
+    SocketDescriptor* socketDescriptor = lookupSocketDescriptor(envir(), streams->fStreamSocketNum);
 
     // Tell it about our subChannel:
     socketDescriptor->registerRTPInterface(streams->fStreamChannelId, this);
   }
 }
 
-Boolean RTPInterface::handleRead(unsigned char* buffer,
-				 unsigned bufferMaxSize,
-				 unsigned& bytesRead,
-				 struct sockaddr_in& fromAddress) {
+Boolean RTPInterface::handleRead(unsigned char* buffer, unsigned bufferMaxSize,
+				 unsigned& bytesRead, struct sockaddr_in& fromAddress, Boolean& packetReadWasIncomplete) {
+  packetReadWasIncomplete = False; // by default
   Boolean readSuccess;
   if (fNextTCPReadStreamSocketNum < 0) {
     // Normal case: read from the (datagram) 'groupsock':
@@ -189,9 +223,9 @@ Boolean RTPInterface::handleRead(unsigned char* buffer,
     // Read from the TCP connection:
     bytesRead = 0;
     unsigned totBytesToRead = fNextTCPReadSize;
-    if (totBytesToRead > bufferMaxSize) totBytesToRead = bufferMaxSize; 
+    if (totBytesToRead > bufferMaxSize) totBytesToRead = bufferMaxSize;
     unsigned curBytesToRead = totBytesToRead;
-    unsigned curBytesRead;
+    int curBytesRead;
     while ((curBytesRead = readSocket(envir(), fNextTCPReadStreamSocketNum,
 				      &buffer[bytesRead], curBytesToRead,
 				      fromAddress)) > 0) {
@@ -199,10 +233,13 @@ Boolean RTPInterface::handleRead(unsigned char* buffer,
       if (bytesRead >= totBytesToRead) break;
       curBytesToRead -= curBytesRead;
     }
-    if (curBytesRead <= 0) {
+    fNextTCPReadSize -= bytesRead;
+    if (curBytesRead == 0 && curBytesToRead > 0) {
+      packetReadWasIncomplete = True;
+      return True;
+    } else if (curBytesRead < 0) {
       bytesRead = 0;
       readSuccess = False;
-      RTPOverTCP_OK = False; // HACK #####
     } else {
       readSuccess = True;
     }
@@ -223,13 +260,7 @@ void RTPInterface::stopNetworkReading() {
   // Also turn off read handling on each of our TCP connections:
   for (tcpStreamRecord* streams = fTCPStreams; streams != NULL;
        streams = streams->fNext) {
-    SocketDescriptor* socketDescriptor
-      = lookupSocketDescriptor(envir(), streams->fStreamSocketNum);
-    if (socketDescriptor != NULL) {
-      socketDescriptor->deregisterRTPInterface(streams->fStreamChannelId);
-        // Note: This may delete "socketDescriptor",
-        // if no more interfaces are using this socket
-    }
+    deregisterSocket(envir(), streams->fStreamSocketNum, streams->fStreamChannelId);
   }
 }
 
@@ -263,15 +294,16 @@ void sendRTPOverTCP(unsigned char* packet, unsigned packetSize,
     return;
   } while (0);
 
-  RTPOverTCP_OK = False; // HACK #####
 #ifdef DEBUG
   fprintf(stderr, "sendRTPOverTCP: failed!\n"); fflush(stderr);
 #endif
 }
 
 SocketDescriptor::SocketDescriptor(UsageEnvironment& env, int socketNum)
-  : fEnv(env), fOurSocketNum(socketNum),
-    fSubChannelHashTable(HashTable::create(ONE_WORD_HASH_KEYS)) {
+  :fEnv(env), fOurSocketNum(socketNum),
+    fSubChannelHashTable(HashTable::create(ONE_WORD_HASH_KEYS)),
+   fServerRequestAlternativeByteHandler(NULL), fServerRequestAlternativeByteHandlerClientData(NULL),
+   fTCPReadingState(AWAITING_DOLLAR) {
 }
 
 SocketDescriptor::~SocketDescriptor() {
@@ -311,52 +343,93 @@ void SocketDescriptor
   }
 }
 
-void SocketDescriptor::tcpReadHandler(SocketDescriptor* socketDescriptor,
-				      int mask) {
-  do {
-    UsageEnvironment& env = socketDescriptor->fEnv; // abbrev
-    int socketNum = socketDescriptor->fOurSocketNum;
+void SocketDescriptor::tcpReadHandler(SocketDescriptor* socketDescriptor, int mask) {
+  socketDescriptor->tcpReadHandler1(mask);
+}
 
-    // Begin by reading and discarding any characters that aren't '$'.
-    // Any such characters are probably regular RTSP responses or
-    // commands from the server.  At present, we can't do anything with
-    // these, because we have taken complete control of reading this socket.
-    // (Later, fix) #####
-    unsigned char c;
-    struct sockaddr_in fromAddress;
-    do {
-      if (readSocket(env, socketNum, &c, 1, fromAddress) != 1) { // error reading TCP socket
-	env.taskScheduler().turnOffBackgroundReadHandling(socketNum); // stops further calls to us
-	return;
+void SocketDescriptor::tcpReadHandler1(int mask) {
+  // We expect the following data over the TCP channel:
+  //   optional RTSP command or response bytes (before the first '$' character)
+  //   a '$' character
+  //   a 1-byte channel id
+  //   a 2-byte packet size (in network byte order)
+  //   the packet data.
+  // However, because the socket is being read asynchronously, this data might arrive in pieces.
+  
+  u_int8_t c;
+  struct sockaddr_in fromAddress;
+  if (fTCPReadingState != AWAITING_PACKET_DATA) {
+    int result = readSocket(fEnv, fOurSocketNum, &c, 1, fromAddress);
+    if (result != 1) { // error reading TCP socket, or no more data available
+      if (result < 0) { // error
+	fEnv.taskScheduler().turnOffBackgroundReadHandling(fOurSocketNum); // stops further calls to us
       }
-    } while (c != '$');
-
-    // The next byte is the stream channel id:
-    unsigned char streamChannelId;
-    if (readSocket(env, socketNum, &streamChannelId, 1, fromAddress)
-	!= 1) break;
-    RTPInterface* rtpInterface
-      = socketDescriptor->lookupRTPInterface(streamChannelId);
-    if (rtpInterface == NULL) break; // we're not interested in this channel
-
-    // The next two bytes are the RTP or RTCP packet size (in network order)
-    unsigned short size;
-    if (readSocketExact(env, socketNum, (unsigned char*)&size, 2,
-			fromAddress) != 2) break;
-    rtpInterface->fNextTCPReadSize = ntohs(size);
-    rtpInterface->fNextTCPReadStreamSocketNum = socketNum;
-    rtpInterface->fNextTCPReadStreamChannelId = streamChannelId;
-#ifdef DEBUG
-    fprintf(stderr, "SocketDescriptor::tcpReadHandler() reading %d bytes on channel %d\n", rtpInterface->fNextTCPReadSize, streamChannelId);
-#endif
-
-    // Now that we have the data set up, call this subchannel's
-    // read handler:
-    if (rtpInterface->fReadHandlerProc != NULL) {
-      rtpInterface->fReadHandlerProc(rtpInterface->fOwner, mask);
+      return;
     }
-
-  } while (0);
+  }
+  
+  switch (fTCPReadingState) {
+    case AWAITING_DOLLAR: {
+      if (c == '$') {
+	fTCPReadingState = AWAITING_STREAM_CHANNEL_ID;
+      } else {
+	// This character is part of a RTSP request or command, which is handled separately:
+	if (fServerRequestAlternativeByteHandler != NULL) {
+	  (*fServerRequestAlternativeByteHandler)(fServerRequestAlternativeByteHandlerClientData, c);
+	}
+      }
+      break;
+    }
+    case AWAITING_STREAM_CHANNEL_ID: {
+      // The byte that we read is the stream channel id.
+      if (lookupRTPInterface(c) != NULL) { // sanity check
+	fStreamChannelId = c;
+	fTCPReadingState = AWAITING_SIZE1;
+      } else {
+	// This wasn't a stream channel id that we expected.  We're (somehow) in a strange state.  Try to recover:
+	fTCPReadingState = AWAITING_DOLLAR;
+      }
+      break;
+    }
+    case AWAITING_SIZE1: {
+      // The byte that we read is the first (high) byte of the 16-bit RTP or RTCP packet 'size'.
+      fSizeByte1 = c;
+      fTCPReadingState = AWAITING_SIZE2;
+      break;
+    }
+    case AWAITING_SIZE2: {
+      // The byte that we read is the second (low) byte of the 16-bit RTP or RTCP packet 'size'.
+      unsigned short size = (fSizeByte1<<8)|c;
+      
+      // Record the information about the packet data that will be read next:
+      RTPInterface* rtpInterface = lookupRTPInterface(fStreamChannelId);
+      if (rtpInterface != NULL) {
+	rtpInterface->fNextTCPReadSize = size;
+	rtpInterface->fNextTCPReadStreamSocketNum = fOurSocketNum;
+	rtpInterface->fNextTCPReadStreamChannelId = fStreamChannelId;
+      }
+      fTCPReadingState = AWAITING_PACKET_DATA;
+      break;
+    }
+    case AWAITING_PACKET_DATA: {
+      // Call the appropriate read handler to get the packet data from the TCP stream:
+      RTPInterface* rtpInterface = lookupRTPInterface(fStreamChannelId);
+      if (rtpInterface != NULL) {
+	if (rtpInterface->fNextTCPReadSize == 0) {
+	  // We've already read all the data for this packet.
+	  fTCPReadingState = AWAITING_DOLLAR;
+	  break;
+	}
+	if (rtpInterface->fReadHandlerProc != NULL) {
+#ifdef DEBUG
+	  fprintf(stderr, "SocketDescriptor::tcpReadHandler() reading %d bytes on channel %d\n", rtpInterface->fNextTCPReadSize, rtpInterface->fNextTCPReadStreamChannelId);
+#endif
+	  rtpInterface->fReadHandlerProc(rtpInterface->fOwner, mask);
+	}
+      }
+      return;
+    }
+  }
 }
 
 
@@ -372,4 +445,4 @@ tcpStreamRecord
 tcpStreamRecord::~tcpStreamRecord() {
   delete fNext;
 }
-                                                                                
+
