@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // A data structure that represents a session that consists of
 // potentially multiple (audio and/or video) sub-sessions
 // (This data structure is used for media *streamers* - i.e., servers.
@@ -183,11 +183,10 @@ Boolean ServerMediaSession::isServerMediaSession() const {
 }
 
 char* ServerMediaSession::generateSDPDescription() {
-  struct in_addr ourIPAddress;
-  ourIPAddress.s_addr = ourSourceAddressForMulticast(envir());
-  char* const ourIPAddressStr
-    = strDup(our_inet_ntoa(ourIPAddress));
-  unsigned ourIPAddressStrSize = strlen(ourIPAddressStr);
+  struct in_addr ipAddress;
+  ipAddress.s_addr = ourIPAddress(envir());
+  char* const ipAddressStr = strDup(our_inet_ntoa(ipAddress));
+  unsigned ipAddressStrSize = strlen(ipAddressStr);
 
   // For a SSM sessions, we need a "a=source-filter: incl ..." line also:
   char* sourceFilterLine;
@@ -195,12 +194,10 @@ char* ServerMediaSession::generateSDPDescription() {
     char const* const sourceFilterFmt =
       "a=source-filter: incl IN IP4 * %s\r\n"
       "a=rtcp-unicast: reflection\r\n";
-    unsigned sourceFilterFmtSize = strlen(sourceFilterFmt)
-      + ourIPAddressStrSize;
+    unsigned const sourceFilterFmtSize = strlen(sourceFilterFmt) + ipAddressStrSize + 1;
 
     sourceFilterLine = new char[sourceFilterFmtSize];
-    sprintf(sourceFilterLine, sourceFilterFmt,
-            ourIPAddressStr);
+    sprintf(sourceFilterLine, sourceFilterFmt, ipAddressStr);
   } else {
     sourceFilterLine = strDup("");
   }
@@ -233,7 +230,7 @@ char* ServerMediaSession::generateSDPDescription() {
     } else { // subsessions have differing durations, so "a=range:" lines go there
       rangeLine = strDup("");
     }
-    
+
     char const* const sdpPrefixFmt =
       "v=0\r\n"
       "o=- %ld%06ld %d IN IP4 %s\r\n"
@@ -249,7 +246,7 @@ char* ServerMediaSession::generateSDPDescription() {
       "a=x-qt-text-inf:%s\r\n"
       "%s";
     sdpLength += strlen(sdpPrefixFmt)
-      + 20 + 6 + 20 + ourIPAddressStrSize
+      + 20 + 6 + 20 + ipAddressStrSize
       + strlen(fDescriptionSDPString)
       + strlen(fInfoSDPString)
       + strlen(libNameStr) + strlen(libVersionStr)
@@ -260,12 +257,12 @@ char* ServerMediaSession::generateSDPDescription() {
       + strlen(fMiscSDPLines);
     sdp = new char[sdpLength];
     if (sdp == NULL) break;
-    
+
     // Generate the SDP prefix (session-level lines):
     sprintf(sdp, sdpPrefixFmt,
 	    fCreationTime.tv_sec, fCreationTime.tv_usec, // o= <session id>
 	    1, // o= <version> // (needs to change if params are modified)
-	    ourIPAddressStr, // o= <address>
+	    ipAddressStr, // o= <address>
 	    fDescriptionSDPString, // s= <description>
 	    fInfoSDPString, // i= <info>
 	    libNameStr, libVersionStr, // a=tool:
@@ -284,7 +281,7 @@ char* ServerMediaSession::generateSDPDescription() {
     }
   } while (0);
 
-  delete[] rangeLine; delete[] sourceFilterLine; delete[] ourIPAddressStr;
+  delete[] rangeLine; delete[] sourceFilterLine; delete[] ipAddressStr;
   return sdp;
 }
 
@@ -342,8 +339,13 @@ void ServerMediaSubsession::pauseStream(unsigned /*clientSessionId*/,
   // default implementation: do nothing
 }
 void ServerMediaSubsession::seekStream(unsigned /*clientSessionId*/,
-				       void* /*streamToken*/, float /*seekNPT*/) {
+				       void* /*streamToken*/, double /*seekNPT*/, double /*streamDuration*/, u_int64_t& numBytes) {
   // default implementation: do nothing
+  numBytes = 0;
+}
+FramedSource* ServerMediaSubsession::getStreamSource(void* /*streamToken*/) {
+  // default implementation: return NULL
+  return NULL;
 }
 void ServerMediaSubsession::setStreamScale(unsigned /*clientSessionId*/,
 					   void* /*streamToken*/, float /*scale*/) {
@@ -386,5 +388,5 @@ ServerMediaSubsession::rangeSDPLine() const {
     char buf[100];
     sprintf(buf, "a=range:npt=0-%.3f\r\n", ourDuration);
     return strDup(buf);
-  }  
+  }
 }
