@@ -11,10 +11,10 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2011 Live Networks, Inc.  All rights reserved.
 // Abstract class for parsing a byte stream
 // Implementation
 
@@ -61,7 +61,7 @@ void StreamParser::ensureValidBytes1(unsigned numBytesNeeded) {
     // Swap banks, but save any still-needed bytes from the old bank:
     unsigned numBytesToSave = fTotNumValidBytes - fSavedParserIndex;
     unsigned char const* from = &curBank()[fSavedParserIndex];
-    
+
     fCurBankNum = (fCurBankNum + 1)%2;
     fCurBank = fBank[fCurBankNum];
     memmove(curBank(), from, numBytesToSave);
@@ -79,7 +79,7 @@ void StreamParser::ensureValidBytes1(unsigned numBytesNeeded) {
 			  << fCurParserIndex << "+ "
 			  << numBytesNeeded << " > "
 			  << BANK_SIZE << ")\n";
-    exit(1);
+    fInputSource->envir().internalError();
   }
 
   // Try to read as many new bytes as will fit in the current bank:
