@@ -11,9 +11,9 @@ more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with this library; if not, write to the Free Software Foundation, Inc.,
-59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 **********/
-// Copyright (c) 1996-2007, Live Networks, Inc.  All rights reserved
+// Copyright (c) 1996-2011, Live Networks, Inc.  All rights reserved
 // A test program that reads a MPEG-1 or 2 Video Elementary Stream file,
 // and streams it using RTP
 // main program
@@ -58,7 +58,7 @@ int main(int argc, char** argv) {
   env = BasicUsageEnvironment::createNew(*scheduler);
 
   // Create 'groupsocks' for RTP and RTCP:
-  char* destinationAddressStr
+  char const* destinationAddressStr
 #ifdef USE_SSM
     = "232.255.42.42";
 #else
@@ -134,6 +134,7 @@ int main(int argc, char** argv) {
 void afterPlaying(void* /*clientData*/) {
   *env << "...done reading from file\n";
 
+  videoSink->stopPlaying();
   Medium::close(videoSource);
 #ifdef SOURCE_IS_PROGRAM_STREAM
   Medium::close(mpegDemux);
@@ -152,7 +153,7 @@ void play() {
 	 << "\" as a byte-stream file source\n";
     exit(1);
   }
-  
+
   FramedSource* videoES;
 #ifdef SOURCE_IS_PROGRAM_STREAM
   // We must demultiplex a Video Elementary Stream from the input source:
